package com.vendor.burn.packagers.nand.ip;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.devices.unknown.UDProperties;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.packagers.nand.BlockType;
import com.vendor.burn.packagers.nand.FlashOption;
import com.vendor.burn.packagers.nand.NandCRandom;
import com.vendor.burn.packagers.nand.PageType;
import com.vendor.burn.packagers.nand.ReverseBlockSizeSector;
import com.vendor.burn.packagers.nand.ReversePageEccSector;
import com.vendor.burn.packagers.nand.ecc.ECC620;
import com.vendor.burn.packagers.nand.ecc.ECCType;
import com.vendor.burn.packagers.nand.ip.NandIP600;
import com.vendor.burn.packagers.nand.ip.NandIPCommon;
import com.vendor.burn.preferences.FastbootPreferences;
import com.vendor.burn.utilitily.BytesUtils;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.io.FileUtils;
import org.eclipse.core.runtime.IProgressMonitor;

public class NandIP620 extends NandIPCommon {
  private static Map<String, Integer> map = new HashMap<>();
  
  private static int NANDIP_620 = 1568;
  
  private static String[] FMCLIST = new String[] { "18", "16", "17", "24", "25", "26", "27", "28", "29" };
  
  private IBurner burner;
  
  private static final ReversePageEccSector[] nfc620_page_ecc_reverse_sector = new ReversePageEccSector[] { 
      new ReversePageEccSector(NANDIP_620, PageType.Size2K, ECCType.Ecc8bit512, 0), new ReversePageEccSector(NANDIP_620, PageType.Size2K, ECCType.Ecc4bit, 1), new ReversePageEccSector(NANDIP_620, PageType.Size2K, ECCType.Ecc4bit, 1), new ReversePageEccSector(NANDIP_620, PageType.Size2K, ECCType.Ecc24bit1k, 2), new ReversePageEccSector(NANDIP_620, PageType.Size4K, ECCType.Ecc8bit512, 3), new ReversePageEccSector(NANDIP_620, PageType.Size4K, ECCType.Ecc4bit, 4), new ReversePageEccSector(NANDIP_620, PageType.Size4K, ECCType.Ecc4bit, 4), new ReversePageEccSector(NANDIP_620, PageType.Size4K, ECCType.Ecc24bit1k, 5), new ReversePageEccSector(NANDIP_620, PageType.Size8K, ECCType.Ecc24bit1k, 6), new ReversePageEccSector(NANDIP_620, PageType.Size8K, ECCType.Ecc40bit1k, 7), 
      new ReversePageEccSector(NANDIP_620, PageType.Size16K, ECCType.Ecc40bit1k, 8), new ReversePageEccSector(NANDIP_620, PageType.Size16K, ECCType.Ecc64bit1k, 9), new ReversePageEccSector(NANDIP_620, PageType.Size8K, ECCType.Ecc64bit1k, 10) };
  
  private static final ReversePageEccSector[] nfc620_page_ecc_reverse_sector_fmc = new ReversePageEccSector[] { new ReversePageEccSector(NANDIP_620, PageType.Size2K, ECCType.Ecc4bit, 0), new ReversePageEccSector(NANDIP_620, PageType.Size2K, ECCType.Ecc24bit1k, 1), new ReversePageEccSector(NANDIP_620, PageType.Size4K, ECCType.Ecc4bit, 2), new ReversePageEccSector(NANDIP_620, PageType.Size4K, ECCType.Ecc24bit1k, 3), new ReversePageEccSector(NANDIP_620, PageType.Size8K, ECCType.Ecc24bit1k, 4), new ReversePageEccSector(NANDIP_620, PageType.Size8K, ECCType.Ecc40bit1k, 5), new ReversePageEccSector(NANDIP_620, PageType.Size16K, ECCType.Ecc40bit1k, 6), new ReversePageEccSector(NANDIP_620, PageType.Size16K, ECCType.Ecc64bit1k, 7), new ReversePageEccSector(NANDIP_620, PageType.Size8K, ECCType.Ecc64bit1k, 8) };
  
  private static final ReverseBlockSizeSector[] nfc620_blocksize_reverse_sector = new ReverseBlockSizeSector[] { new ReverseBlockSizeSector(NANDIP_620, BlockType.BlockSize64, 0), new ReverseBlockSizeSector(NANDIP_620, BlockType.BlockSize128, 1), new ReverseBlockSizeSector(NANDIP_620, BlockType.BlockSize256, 2), new ReverseBlockSizeSector(NANDIP_620, BlockType.BlockSize512, 3) };
  
  private String boardType;
  
  public NandIP620() {
    map.put("2k4bit", Integer.valueOf(60));
    map.put("2k8bit", Integer.valueOf(64));
    map.put("2k13bit", Integer.valueOf(64));
    map.put("2k24bit", Integer.valueOf(116));
    map.put("2k27bit", Integer.valueOf(128));
    map.put("4k4bit", Integer.valueOf(88));
    map.put("4k8bit", Integer.valueOf(128));
    map.put("4k13bit", Integer.valueOf(112));
    map.put("4k24bit", Integer.valueOf(200));
    map.put("4k27bit", Integer.valueOf(224));
    map.put("8k24bit", Integer.valueOf(368));
    map.put("8k27bit", Integer.valueOf(416));
    map.put("8k40bit", Integer.valueOf(592));
    map.put("8k41bit", Integer.valueOf(608));
    map.put("8k60bit", Integer.valueOf(896));
    map.put("8k64bit", Integer.valueOf(928));
    map.put("8k80bit", Integer.valueOf(1152));
    map.put("16k40bit", Integer.valueOf(1152));
    map.put("16k41bit", Integer.valueOf(1184));
    map.put("16k60bit", Integer.valueOf(1760));
    map.put("16k64bit", Integer.valueOf(1824));
    map.put("16k80bit", Integer.valueOf(1376));
  }
  
  public FlashOption[] getAvailableOptions(IBurner burner) {
    this.burner = burner;
    UDProperties properties = ((UnknownDevice)burner.getBurnContext().getDeviceSetting()).getProperties();
    this.boardType = properties.queryForString("TYPE");
    if (this.boardType != null && this.boardType.equalsIgnoreCase("CAM"))
      return new FlashOption[] { new FlashOption(ECCType.Ecc4bit, PageType.Size2K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size2K, -1), new FlashOption(ECCType.Ecc4bit, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc40bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc64bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc40bit1k, PageType.Size16K, -1), new FlashOption(ECCType.Ecc64bit1k, PageType.Size16K, -1) }; 
    if (isFMC())
      return new FlashOption[] { 
          new FlashOption(ECCType.Ecc4bit, PageType.Size2K, -1), new FlashOption(ECCType.Ecc8bit512, PageType.Size2K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size2K, -1), new FlashOption(ECCType.Ecc4bit, PageType.Size4K, -1), new FlashOption(ECCType.Ecc8bit512, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc40bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc64bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc40bit1k, PageType.Size16K, -1), 
          new FlashOption(ECCType.Ecc64bit1k, PageType.Size16K, -1) }; 
    return new FlashOption[] { new FlashOption(ECCType.Ecc4bit, PageType.Size2K, -1), new FlashOption(ECCType.Ecc8bit512, PageType.Size2K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size2K, -1), new FlashOption(ECCType.Ecc4bit, PageType.Size4K, -1), new FlashOption(ECCType.Ecc8bit512, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size4K, -1), new FlashOption(ECCType.Ecc24bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc40bit1k, PageType.Size8K, -1), new FlashOption(ECCType.Ecc40bit1k, PageType.Size16K, -1) };
  }
  
  public byte[] genPageData(byte[] pagebuf, FlashOption option, int oobSize) {
    ECC620 ecc = new ECC620();
    if (option.getEccType() == ECCType.Ecc4bit) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 14); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 14); 
    } 
    if (option.getEccType() == ECCType.Ecc8bit512) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 6, 28); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 14, 28); 
    } 
    if (option.getEccType() == ECCType.Ecc24bit1k) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 42); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 42); 
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 42); 
    } 
    if (option.getEccType() == ECCType.Ecc40bit1k) {
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 70); 
      if (option.getPageType() == PageType.Size16K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 70); 
    } 
    if (option.getEccType() == ECCType.Ecc64bit1k) {
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 112); 
      if (option.getPageType() == PageType.Size16K)
        return genPageDataBytesFor620(ecc, option, pagebuf, 30, 112); 
    } 
    return null;
  }
  
  public void make(String outputPath, FlashOption option, List<PartitionItem> partitions, int blockSize, boolean isSelectRandom, IFastbootProgressMonitor fastbootMonitor, IProgressMonitor monitor) {
    int usedOobSize = 0;
    usedOobSize = getUsedOobSize(option);
    fastbootMonitor.infoln("\nStart to make nand burner image...");
    fastbootMonitor.infoln("Options: " + option.toString() + "\n");
    monitor.beginTask(Messages.NandImageDialog_Title, partitions.size());
    int randomPageNum = -1;
    if (isSelectRandom && 
      blockSize > -1)
      if (option.getPageType().getPageSize() == PageType.Size16K.getPageSize()) {
        randomPageNum = blockSize / option.getPageType().getPageSize();
      } else {
        randomPageNum = blockSize / 8192;
      }  
    for (PartitionItem item : partitions) {
      if (monitor.isCanceled())
        return; 
      boolean itemState = (!item.isChecked() || item.getFile() == null);
      if (itemState) {
        monitor.worked(1);
        continue;
      } 
      differenceBlockSize(item, blockSize);
      fastbootMonitor.info("Make image for '" + item.getFile().getName() + "'...");
      int pageCheckSize = option.getPageType().getPageSize();
      if (item.getFileSystem() == FileSystem.yaffs)
        if (this.boardType != null && this.boardType.equalsIgnoreCase("CAM")) {
          pageCheckSize = option.getPageType().getPageSize() + option.getOOBSize();
        } else {
          pageCheckSize = option.getPageType().getPageSize() + 32;
        }  
      File file = (item.getNewTempFile() == null) ? item.getFile() : item.getNewTempFile();
      int pageCount = (int)((file.length() + pageCheckSize - 1L) / pageCheckSize);
      FileSystem fileSystem = item.getFileSystem();
      if (fileSystem == FileSystem.yaffs)
        pageCount = (int)(file.length() / pageCheckSize); 
      File outFile = FileUtils.getFile(new String[] { outputPath + outputPath + File.separator + ".burn" });
      FileUtils.deleteFile(outFile);
      RandomAccessFile inRanFile = null;
      RandomAccessFile outRanFile = null;
      boolean ret = true;
      try {
        FileUtils.checkFilePath(file);
        inRanFile = new RandomAccessFile(file, "r");
        outRanFile = new RandomAccessFile(outFile, "rw");
        int pageMakeSize = option.getPageType().getPageSize() + 32;
        if (this.boardType != null && this.boardType.equalsIgnoreCase("CAM"))
          pageMakeSize = option.getPageType().getPageSize() + option.getOOBSize(); 
        long start = item.getStart();
        int reversePageEccEec = 0;
        int reverseBlockEec = 0;
        boolean page0ReverseFlag = false;
        boolean page1ReverseFlag = false;
        boolean nandPinSaveMpde = false;
        if (ToolUtils.isCmdLine()) {
          nandPinSaveMpde = PreferenceConfig.isImgOpenPinSaveMode();
        } else {
          nandPinSaveMpde = FastbootPreferences.isNandPinSaveMpde();
        } 
        String curPinSaveMode = null;
        boolean supportEmpty = false;
        if (this.curSelectedNandWay != null)
          if (this.curSelectedNandWay.equals("Nand")) {
            curPinSaveMode = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getNandPinSaveMode();
            supportEmpty = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportNandEmpty();
          } else if (this.curSelectedNandWay.equals("SPINand")) {
            curPinSaveMode = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getSpiNandPinSaveMode();
            supportEmpty = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportSPINandEmpty();
          }  
        if (nandPinSaveMpde)
          if (start == 0L) {
            ReversePageEccSector[] sector = null;
            if (curPinSaveMode != null && curPinSaveMode
              .equalsIgnoreCase(getPinSaveModeArray()[0])) {
              if (isFMC()) {
                sector = nfc620_page_ecc_reverse_sector_fmc;
              } else {
                sector = nfc620_page_ecc_reverse_sector;
              } 
            } else if (curPinSaveMode != null && curPinSaveMode
              .equalsIgnoreCase(getPinSaveModeArray()[1])) {
              sector = (ReversePageEccSector[])NandIP600.fmc_page_ecc_reverse_sector.clone();
            } else if (isFMC()) {
              sector = (ReversePageEccSector[])nfc620_page_ecc_reverse_sector_fmc.clone();
            } else {
              sector = (ReversePageEccSector[])nfc620_page_ecc_reverse_sector.clone();
            } 
            PageType pageType = option.getPageType();
            ECCType eccType = option.getEccType();
            ReversePageEccSector pageSizeEccReverseSector = getPageSizeEccReverseSector(pageType, eccType, sector);
            if (pageSizeEccReverseSector != null)
              reversePageEccEec = pageSizeEccReverseSector.getReverse_sec(); 
            ReverseBlockSizeSector blockSizeReverseSector = getBlockSizeReverseSector(blockSize, pageType);
            if (blockSizeReverseSector != null)
              reverseBlockEec = blockSizeReverseSector.getReverse_sec(); 
          }  
        ret = writeDataToFile(option, fastbootMonitor, usedOobSize, randomPageNum, pageCheckSize, pageCount, fileSystem, inRanFile, outRanFile, ret, pageMakeSize, start, reversePageEccEec, reverseBlockEec, page0ReverseFlag, page1ReverseFlag, nandPinSaveMpde);
        if (monitor.isCanceled())
          return; 
        if (supportEmpty)
          ret = checkEmptyFlagAndWriteToFile(option, blockSize, fastbootMonitor, usedOobSize, randomPageNum, pageCheckSize, pageCount, fileSystem, outRanFile, ret, pageMakeSize); 
      } catch (FileNotFoundException e) {
        fastbootMonitor.warnln("FAILED! (Source file not found.)");
        monitor.worked(1);
        continue;
      } catch (IOException iOException) {
      
      } finally {
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } 
      fastbootMonitor.info("OK!");
      fastbootMonitor.warnln("(Start Address = 0x" + 
          Long.toHexString(item.getStart()) + ", Length = 0x" + 
          Long.toHexString(item.getLength()) + ")");
      monitor.worked(1);
    } 
    fastbootMonitor
      .infoln("\nImage creating operation ends. All of the selected partitions are processed.");
    monitor.done();
  }
  
  private boolean writeDataToFile(FlashOption option, IFastbootProgressMonitor fastbootMonitor, int usedOobSize, int randomPageNum, int pageCheckSize, int pageCount, FileSystem fileSystem, RandomAccessFile inRanFile, RandomAccessFile outRanFile, boolean ret, int pageMakeSize, long start, int reversePageEccEec, int reverseBlockEec, boolean page0ReverseFlag, boolean page1ReverseFlag, boolean nandPinSaveMpde) {
    for (int i = 0; i < pageCount; i++) {
      byte[] pagebuf = BytesUtils.genBytesWithFF(pageMakeSize);
      try {
        inRanFile.seek((i * pageCheckSize));
        int read = inRanFile.read(pagebuf, 0, pageCheckSize);
        if (read < 0)
          System.out.println("Read Error."); 
        boolean isCAM = (this.boardType != null && this.boardType.equalsIgnoreCase("CAM"));
        pageBufferValue(option, pageCheckSize, fileSystem, pageMakeSize, pagebuf, isCAM);
      } catch (IOException e) {
        fastbootMonitor.warnln("FAILED! (Read data error.)");
        ret = false;
        break;
      } 
      byte[] data = null;
      try {
        data = genPageData(pagebuf, option, option.getOOBSize());
        if (data == null)
          break; 
        dealWithNandCRandom(option, usedOobSize, randomPageNum, i, data);
        if (nandPinSaveMpde)
          if (start == 0L) {
            if (!page0ReverseFlag) {
              int page0ReverseOff = 32 * reversePageEccEec;
              reverseBuf(data, page0ReverseOff, 32);
            } 
            if (page0ReverseFlag && !page1ReverseFlag) {
              int page1ReverseOff = 32 * reverseBlockEec;
              reverseBuf(data, page1ReverseOff, 32);
              page1ReverseFlag = true;
            } 
            page0ReverseFlag = true;
          }  
        outRanFile.seek((i * (option.getPageType().getPageSize() + option.getOOBSize())));
        outRanFile.write(data);
        if (outRanFile.length() % (option.getPageType().getPageSize() + option.getOOBSize()) != 0L) {
          fastbootMonitor.warnln("FAILED! (Could not write output file.)");
          ret = false;
          break;
        } 
      } catch (Exception e) {
        fastbootMonitor.warnln("FAILED! (Data convert error.)");
        ret = false;
        break;
      } 
    } 
    return ret;
  }
  
  public void dealWithNandCRandom(FlashOption option, int usedOobSize, int randomPageNum, int i, byte[] data) {
    if (randomPageNum > 0) {
      NandCRandom random = new NandCRandom();
      if (usedOobSize != 0) {
        random.page_random_gen610(data, option.getPageType().getPageSize(), usedOobSize, i % randomPageNum);
      } else {
        random.page_random_gen610(data, option.getPageType().getPageSize(), option
            .getOOBSize(), i % randomPageNum);
      } 
    } 
  }
  
  private boolean checkEmptyFlagAndWriteToFile(FlashOption option, int blockSize, IFastbootProgressMonitor fastbootMonitor, int usedOobSize, int randomPageNum, int pageCheckSize, int pageCount, FileSystem fileSystem, RandomAccessFile outRanFile, boolean ret, int pageMakeSize) {
    int currentPageIndex = pageCount;
    int pageNumInclData = pageCount;
    int pagePerBlock = blockSize / option.getPageType().getPageSize();
    int pageNumTotalBlock = pageNumInclData & (pagePerBlock - 1 ^ 0xFFFFFFFF);
    pageNumTotalBlock += ((pageNumInclData & pagePerBlock - 1) > 0) ? pagePerBlock : 0;
    int pageNumFillNull = 0;
    if (pageNumTotalBlock - pageNumInclData > 0)
      pageNumFillNull = pageNumTotalBlock - pageNumInclData; 
    while (pageNumFillNull-- > 0) {
      byte[] pagebuf = BytesUtils.genBytesWithFF(pageMakeSize);
      boolean isCAM = (this.boardType != null && this.boardType.equalsIgnoreCase("CAM"));
      pageBufferValue(option, pageCheckSize, fileSystem, pageMakeSize, pagebuf, isCAM);
      byte[] data = null;
      try {
        data = genPageData(pagebuf, option, option.getOOBSize());
        NandCRandom random = new NandCRandom();
        if (randomPageNum > -1 && data != null) {
          if (usedOobSize != 0) {
            random.page_random_gen610(data, option.getPageType().getPageSize(), usedOobSize, currentPageIndex % randomPageNum);
          } else {
            random.page_random_gen610(data, option.getPageType().getPageSize(), option
                .getOOBSize(), currentPageIndex % randomPageNum);
          } 
          outRanFile.write(data);
        } 
        currentPageIndex++;
      } catch (Exception e) {
        fastbootMonitor.warnln("FAILED! (Write Empty flag error.)");
        ret = false;
        break;
      } 
    } 
    return ret;
  }
  
  public void pageBufferValue(FlashOption option, int pageCheckSize, FileSystem fileSystem, int pageMakeSize, byte[] pagebuf, boolean isCAM) {
    if (isCAM) {
      int camPageEmptyFlag = option.getPageType().getPageSize() + 32;
      if (fileSystem == FileSystem.ubiFs) {
        if (checkBufferFF(pagebuf, pageCheckSize)) {
          pagebuf[camPageEmptyFlag - 1] = 0;
          pagebuf[camPageEmptyFlag - 2] = 0;
        } 
      } else {
        pagebuf[camPageEmptyFlag - 1] = 0;
        pagebuf[camPageEmptyFlag - 2] = 0;
      } 
    } else if (fileSystem == FileSystem.ubiFs) {
      if (checkBufferFF(pagebuf, pageCheckSize)) {
        pagebuf[pageMakeSize - 1] = 0;
        pagebuf[pageMakeSize - 2] = 0;
      } 
    } else {
      pagebuf[pageMakeSize - 1] = 0;
      pagebuf[pageMakeSize - 2] = 0;
    } 
  }
  
  public boolean checkBufferFF(byte[] pagebuf, int length) {
    for (int index = 0; index < length; index++) {
      if (pagebuf[index] != -1)
        return true; 
    } 
    return false;
  }
  
  public static void printHexString(byte[] b) {
    for (int i = 0; i < b.length; i++) {
      String hex = Integer.toHexString(b[i] & 0xFF);
      if (hex.length() == 1)
        hex = "0" + hex; 
      System.out.print(hex.toUpperCase() + ",");
    } 
  }
  
  private static void reverseBuf(byte[] buf, int startOff, int reverseLen) {
    for (int i = startOff; i < startOff + reverseLen; i++)
      buf[i] = (byte)(buf[i] ^ 0xFFFFFFFF); 
  }
  
  private ReversePageEccSector getPageSizeEccReverseSector(PageType pagetype, ECCType ecctype, ReversePageEccSector[] sector) {
    for (int i = 0; i < sector.length; i++) {
      if (sector[i].getEcctype() == ecctype && sector[i]
        .getPagetype() == pagetype)
        return sector[i]; 
    } 
    return null;
  }
  
  private ReverseBlockSizeSector getBlockSizeReverseSector(int blocksize, PageType pageType) {
    int pageNum = blocksize / pageType.getPageSize();
    ReverseBlockSizeSector[] sector = nfc620_blocksize_reverse_sector;
    for (int i = 0; i < sector.length; i++) {
      if (sector[i].getBlocktype().getSize() == pageNum)
        return sector[i]; 
    } 
    return null;
  }
  
  private int getUsedOobSize(FlashOption option) {
    int usedOobSize = 0;
    if (option.getEccType() == ECCType.Ecc4bit)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k4bit") != null)
          return ((Integer)map.get("2k4bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K && 
        map.get("4k4bit") != null) {
        return ((Integer)map.get("4k4bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc8bit512)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k8bit") != null)
          return ((Integer)map.get("2k8bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K && 
        map.get("4k8bit") != null) {
        return ((Integer)map.get("4k8bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc13bit)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k13bit") != null)
          return ((Integer)map.get("2k13bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K && 
        map.get("4k13bit") != null) {
        return ((Integer)map.get("4k13bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc24bit1k)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k24bit") != null)
          return ((Integer)map.get("2k24bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K) {
        if (map.get("4k24bit") != null)
          return ((Integer)map.get("4k24bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size8K && 
        map.get("8k24bit") != null) {
        return ((Integer)map.get("8k24bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc27bit)
      if (option.getPageType() == PageType.Size2K) {
        if (map.get("2k27bit") != null)
          return ((Integer)map.get("2k27bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size4K) {
        if (map.get("4k27bit") != null)
          return ((Integer)map.get("4k27bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size8K && 
        map.get("8k27bit") != null) {
        return ((Integer)map.get("8k27bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc40bit1k)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k40bit") != null)
          return ((Integer)map.get("8k40bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k40bit") != null) {
        return ((Integer)map.get("16k40bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc41bit)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k41bit") != null)
          return ((Integer)map.get("8k41bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k41bit") != null) {
        return ((Integer)map.get("16k41bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc60bit)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k60bit") != null)
          return ((Integer)map.get("8k60bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k60bit") != null) {
        return ((Integer)map.get("16k60bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc64bit1k)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k64bit") != null)
          return ((Integer)map.get("8k64bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k64bit") != null) {
        return ((Integer)map.get("16k64bit")).intValue();
      }  
    if (option.getEccType() == ECCType.Ecc80bit)
      if (option.getPageType() == PageType.Size8K) {
        if (map.get("8k80bit") != null)
          return ((Integer)map.get("8k80bit")).intValue(); 
      } else if (option.getPageType() == PageType.Size16K && 
        map.get("16k80bit") != null) {
        return ((Integer)map.get("16k80bit")).intValue();
      }  
    return usedOobSize;
  }
  
  public String toString() {
    return "Nand IP 620";
  }
  
  private boolean isFMC() {
    String chipName = ToolUtils.getChipNameType();
    if (chipName == null)
      return false; 
    if (chipName.contains("-"))
      chipName = chipName.replace("-", "_"); 
    for (String name : FMCLIST) {
      if (chipName.equalsIgnoreCase(name))
        return true; 
    } 
    return false;
  }
  
  public static ReversePageEccSector[] getNfc620PageEccReverseSector() {
    return (ReversePageEccSector[])nfc620_page_ecc_reverse_sector.clone();
  }
}
